@@ -292,6 +292,7 @@ DEFAULT_EMPTY_DB_ID = '__default_empty_db__'
 
 class TestDataBaseHandler(object):
     DRIVER = None
+
     db_cache = {}
     explored_glob = set()
 
@@ -535,6 +536,10 @@ class NoCreateDropDatabaseHandler(TestDataBaseHandler):
 class PostgresTestDataBaseHandler(TestDataBaseHandler):
     DRIVER = 'postgres'
 
+    # Separate db_cache for PG databases, to avoid collisions with sqlite dbs
+    db_cache = {}
+    explored_glob = set()
+
     __CTL = set()
 
     @classmethod
@@ -560,8 +565,12 @@ class PostgresTestDataBaseHandler(TestDataBaseHandler):
         env = os.environ.copy()
         env['PGPORT'] = str(port)
         env['PGHOST'] = str(directory)
+        options = '-h "" -k %s -p %s' % (directory, port)
+        options += ' -c fsync=off -c full_page_writes=off'
+        options += ' -c synchronous_commit=off'
         try:
-            subprocess.check_call(['pg_ctl', 'start', '-w', '-D', datadir, '-o', '-h "" -k %s -p %s' % (directory, port)],
+            subprocess.check_call(['pg_ctl', 'start', '-w', '-D', datadir,
+                                   '-o', options],
                                   env=env)
         except OSError, err:
             if err.errno == errno.ENOENT:
@@ -670,12 +679,14 @@ class PostgresTestDataBaseHandler(TestDataBaseHandler):
             backup_name = self._backup_name(db_id)
             self._drop(backup_name)
             self.system_source['db-name'] = backup_name
-            # during postgres database initialization, there is no repo set here.
-            assert self._repo is None
-            #self._repo.turn_repo_off()
-            createdb(self.helper, self.system_source, self.dbcnx, self.cursor, template=orig_name)
-            self.dbcnx.commit()
-            #self._repo.turn_repo_on()
+            if self._repo:
+                self._repo.turn_repo_off()
+            try:
+                createdb(self.helper, self.system_source, self.dbcnx, self.cursor, template=orig_name)
+                self.dbcnx.commit()
+            finally:
+                if self._repo:
+                    self._repo.turn_repo_on()
             return backup_name
         finally:
             self.system_source['db-name'] = orig_name
