@@ -31,6 +31,7 @@ from logilab.common.decorators import cached
 from logilab.common.configuration import REQUIRED, read_old_config
 from logilab.common.shellutils import ASK
 from logilab.common.changelog import Version
+from logilab.common.deprecation import deprecated
 
 from cubicweb import ConfigurationError, ExecutionError
 from cubicweb.cwconfig import CubicWebConfiguration as cwcfg
@@ -247,12 +248,13 @@ class MigrationHelper(object):
         local_ctx = self._create_context()
         try:
             import readline
-            from rlcompleter import Completer
+            from cubicweb.toolsutils import CWShellCompleter
         except ImportError:
             # readline not available
             pass
         else:
-            readline.set_completer(Completer(local_ctx).complete)
+            rql_completer = CWShellCompleter(local_ctx)
+            readline.set_completer(rql_completer.complete)
             readline.parse_and_bind('tab: complete')
             home_key = 'HOME'
             if sys.platform == 'win32':
@@ -406,7 +408,11 @@ type "exit" or Ctrl-D to quit the shell and resume operation"""
             self.config.add_cubes(newcubes)
         return newcubes
 
+    @deprecated('[3.20] use drop_cube() instead of remove_cube()')
     def cmd_remove_cube(self, cube, removedeps=False):
+        return self.cmd_drop_cube(cube, removedeps)
+
+    def cmd_drop_cube(self, cube, removedeps=False):
         if removedeps:
             toremove = self.config.expand_cubes([cube])
         else:
