@@ -338,8 +338,7 @@ class Repository(object):
             except Exception as ex:
                 import traceback
                 traceback.print_exc()
-                raise (Exception('Is the database initialised ? (cause: %s)' % ex),
-                       None, sys.exc_info()[-1])
+                raise Exception('Is the database initialised ? (cause: %s)' % ex)
         return appschema
 
     def _prepare_startup(self):
@@ -651,8 +650,8 @@ class Repository(object):
                                    query_attrs)
             return rset.rows
 
-    def connect(self, login, **kwargs):
-        """open a session for a given user
+    def new_session(self, login, **kwargs):
+        """open a new session for a given user
 
         raise `AuthenticationError` if the authentication failed
         raise `ConnectionError` if we can't open a connection
@@ -678,7 +677,11 @@ class Repository(object):
             # commit connection at this point in case write operation has been
             # done during `session_open` hooks
             cnx.commit()
-        return session.sessionid
+        return session
+
+    def connect(self, login, **kwargs):
+        """open a new session for a given user and return its sessionid """
+        return self.new_session(login, **kwargs).sessionid
 
     def execute(self, sessionid, rqlstring, args=None, build_descr=True,
                 txid=None):
@@ -1161,7 +1164,7 @@ class Repository(object):
     def glob_add_entity(self, cnx, edited):
         """add an entity to the repository
 
-        the entity eid should originaly be None and a unique eid is assigned to
+        the entity eid should originally be None and a unique eid is assigned to
         the entity instance
         """
         entity = edited.entity
