@@ -24,6 +24,7 @@ __docformat__ = 'restructuredtext en'
 # completion). So import locally in command helpers.
 import sys
 import os
+from contextlib import contextmanager
 import logging
 import subprocess
 
@@ -31,6 +32,8 @@ from logilab.common import nullobject
 from logilab.common.configuration import Configuration, merge_options
 from logilab.common.shellutils import ASK, generate_password
 
+from logilab.database import get_db_helper, get_connection
+
 from cubicweb import AuthenticationError, ExecutionError, ConfigurationError
 from cubicweb.toolsutils import Command, CommandHandler, underline_title
 from cubicweb.cwctl import CWCTL, check_options_consistency, ConfigureInstanceCommand
@@ -47,7 +50,6 @@ def source_cnx(source, dbname=None, special_privs=False, interactive=True):
     given server.serverconfig
     """
     from getpass import getpass
-    from logilab.database import get_connection, get_db_helper
     dbhost = source.get('db-host')
     if dbname is None:
         dbname = source['db-name']
@@ -86,6 +88,7 @@ def source_cnx(source, dbname=None, special_privs=False, interactive=True):
     extra = extra_args and {'extra_args': extra_args} or {}
     cnx = get_connection(driver, dbhost, dbname, user, password=password,
                          port=source.get('db-port'),
+                         schema=source.get('db-namespace'),
                          **extra)
     try:
         cnx.logged_user = user
@@ -104,7 +107,6 @@ def system_source_cnx(source, dbms_system_base=False,
     create/drop the instance database)
     """
     if dbms_system_base:
-        from logilab.database import get_db_helper
         system_db = get_db_helper(source['db-driver']).system_database()
         return source_cnx(source, system_db, special_privs=special_privs,
                           interactive=interactive)
@@ -116,7 +118,6 @@ def _db_sys_cnx(source, special_privs, interactive=True):
     database)
     """
     import logilab.common as lgp
-    from logilab.database import get_db_helper
     lgp.USE_MX_DATETIME = False
     driver = source['db-driver']
     helper = get_db_helper(driver)
@@ -205,56 +206,99 @@ class RepositoryCreateHandler(CommandHandler):
             print ('-> nevermind, you can do it later with '
                    '"cubicweb-ctl db-create %s".' % self.config.appid)
 
-ERROR = nullobject()
 
-def confirm_on_error_or_die(msg, func, *args, **kwargs):
+@contextmanager
+def db_transaction(source, privilege):
+    """Open a transaction to the instance database"""
+    cnx = system_source_cnx(source, special_privs=privilege)
+    cursor = cnx.cursor()
     try:
-        return func(*args, **kwargs)
-    except Exception as ex:
-        print 'ERROR', ex
-        if not ASK.confirm('An error occurred while %s. Continue anyway?' % msg):
-            raise ExecutionError(str(ex))
-    return ERROR
+        yield cursor
+    except:
+        cnx.rollback()
+        cnx.close()
+        raise
+    else:
+        cnx.commit()
+        cnx.close()
+
+
+@contextmanager
+def db_sys_transaction(source, privilege):
+    """Open a transaction to the system database"""
+    cnx = _db_sys_cnx(source, privilege)
+    cursor = cnx.cursor()
+    try:
+        yield cursor
+    except:
+        cnx.rollback()
+        cnx.close()
+        raise
+    else:
+        cnx.commit()
+        cnx.close()
+
 
 class RepositoryDeleteHandler(CommandHandler):
     cmdname = 'delete'
     cfgname = 'repository'
 
+    def _drop_namespace(self, source):
+        db_namespace = source.get('db-namespace')
+        with db_transaction(source, privilege='DROP SCHEMA') as cursor:
+            helper = get_db_helper(source['db-driver'])
+            helper.drop_schema(cursor, db_namespace)
+            print '-> database schema %s dropped' % db_namespace
+
+    def _drop_database(self, source):
+        dbname = source['db-name']
+        if source['db-driver'] == 'sqlite':
+            print 'deleting database file %(db-name)s' % source
+            os.unlink(source['db-name'])
+            print '-> database %(db-name)s dropped.' % source
+        else:
+            helper = get_db_helper(source['db-driver'])
+            with db_sys_transaction(source, privilege='DROP DATABASE') as cursor:
+                print 'dropping database %(db-name)s' % source
+                cursor.execute('DROP DATABASE "%(db-name)s"' % source)
+                print '-> database %(db-name)s dropped.' % source
+
+    def _drop_user(self, source):
+        user = source['db-user'] or None
+        if user is not None:
+            with db_sys_transaction(source, privilege='DROP USER') as cursor:
+                print 'dropping user %s' % user
+                cursor.execute('DROP USER %s' % user)
+
+    def _cleanup_steps(self, source):
+        # 1/ delete namespace if used
+        db_namespace = source.get('db-namespace')
+        if db_namespace:
+            yield ('Delete database namespace "%s"' % db_namespace,
+                   self._drop_namespace, True)
+        # 2/ delete database
+        yield ('Delete database "%(db-name)s"' % source,
+               self._drop_database, True)
+        # 3/ delete user
+        helper = get_db_helper(source['db-driver'])
+        if source['db-user'] and helper.users_support:
+            # XXX should check we are not connected as user
+            yield ('Delete user "%(db-user)s"' % source,
+                   self._drop_user, False)
+
     def cleanup(self):
         """remove instance's configuration and database"""
-        from logilab.database import get_db_helper
         source = self.config.system_source_config
-        dbname = source['db-name']
-        helper = get_db_helper(source['db-driver'])
-        if ASK.confirm('Delete database %s ?' % dbname):
-            if source['db-driver'] == 'sqlite':
-                if confirm_on_error_or_die(
-                    'deleting database file %s' % dbname,
-                    os.unlink, source['db-name']) is not ERROR:
-                    print '-> database %s dropped.' % dbname
-                return
-            user = source['db-user'] or None
-            cnx = confirm_on_error_or_die('connecting to database %s' % dbname,
-                                          _db_sys_cnx, source, 'DROP DATABASE')
-            if cnx is ERROR:
-                return
-            cursor = cnx.cursor()
-            try:
-                if confirm_on_error_or_die(
-                    'dropping database %s' % dbname,
-                    cursor.execute, 'DROP DATABASE "%s"' % dbname) is not ERROR:
-                    print '-> database %s dropped.' % dbname
-                # XXX should check we are not connected as user
-                if user and helper.users_support and \
-                       ASK.confirm('Delete user %s ?' % user, default_is_yes=False):
-                    if confirm_on_error_or_die(
-                        'dropping user %s' % user,
-                        cursor.execute, 'DROP USER %s' % user) is not ERROR:
-                        print '-> user %s dropped.' % user
-                cnx.commit()
-            except BaseException:
-                cnx.rollback()
-                raise
+        for msg, step, default in self._cleanup_steps(source):
+            if ASK.confirm(msg, default_is_yes=default):
+                try:
+                    step(source)
+                except Exception as exc:
+                    print 'ERROR', exc
+                    if ASK.confirm('An error occurred. Continue anyway?',
+                                   default_is_yes=False):
+                        continue
+                    raise ExecutionError(str(exc))
 
 
 class RepositoryStartHandler(CommandHandler):
@@ -294,6 +338,7 @@ def createdb(helper, source, dbcnx, cursor, **kwargs):
         helper.create_database(cursor, source['db-name'],
                                dbencoding=source['db-encoding'], **kwargs)
 
+
 class CreateInstanceDBCommand(Command):
     """Create the system database of an instance (run after 'create').
 
@@ -331,7 +376,6 @@ class CreateInstanceDBCommand(Command):
 
     def run(self, args):
         """run the command with its specific arguments"""
-        from logilab.database import get_db_helper
         check_options_consistency(self.config)
         automatic = self.get('automatic')
         appid = args.pop()
@@ -371,10 +415,14 @@ class CreateInstanceDBCommand(Command):
             except BaseException:
                 dbcnx.rollback()
                 raise
-        cnx = system_source_cnx(source, special_privs='CREATE LANGUAGE',
+        cnx = system_source_cnx(source, special_privs='CREATE LANGUAGE/SCHEMA',
                                 interactive=not automatic)
         cursor = cnx.cursor()
         helper.init_fti_extensions(cursor)
+        namespace = source.get('db-namespace')
+        if namespace and ASK.confirm('Create schema %s in database %s ?'
+                                     % (namespace, dbname)):
+            helper.create_schema(cursor, namespace)
         cnx.commit()
         # postgres specific stuff
         if driver == 'postgres':
@@ -439,7 +487,6 @@ class InitInstanceCommand(Command):
         check_options_consistency(self.config)
         print '\n'+underline_title('Initializing the system database')
         from cubicweb.server import init_repository
-        from logilab.database import get_connection
         appid = args[0]
         config = ServerConfiguration.config_for(appid)
         try:
@@ -450,7 +497,7 @@ class InitInstanceCommand(Command):
                 system['db-driver'], database=system['db-name'],
                 host=system.get('db-host'), port=system.get('db-port'),
                 user=system.get('db-user') or '', password=system.get('db-password') or '',
-                **extra)
+                schema=system.get('db-namespace'), **extra)
         except Exception as ex:
             raise ConfigurationError(
                 'You seem to have provided wrong connection information in '\
@@ -482,7 +529,6 @@ class AddSourceCommand(Command):
     def run(self, args):
         appid = args[0]
         config = ServerConfiguration.config_for(appid)
-        config.quick_start = True
         repo, cnx = repo_cnx(config)
         with cnx:
             used = set(n for n, in cnx.execute('Any SN WHERE S is CWSource, S name SN'))
@@ -504,6 +550,12 @@ class AddSourceCommand(Command):
                                'not used by the instance.')
                         continue
                 break
+            while True:
+                parser = raw_input('parser type (%s): '
+                                    % ', '.join(sorted(repo.vreg['parsers'])))
+                if parser in repo.vreg['parsers']:
+                    break
+                print '-> unknown parser identifier, use one of the available types.'
             while True:
                 sourceuri = raw_input('source identifier (a unique name used to '
                                       'tell sources apart): ').strip()
@@ -515,11 +567,13 @@ class AddSourceCommand(Command):
                         print '-> uri already used, choose another one.'
                     else:
                         break
+            url = raw_input('source URL (leave empty for none): ').strip()
+            url = unicode(url) if url else None
             # XXX configurable inputlevel
             sconfig = ask_source_config(config, type, inputlevel=self.config.config_level)
             cfgstr = unicode(generate_source_config(sconfig), sys.stdin.encoding)
-            cnx.create_entity('CWSource', name=sourceuri,
-                              type=unicode(type), config=cfgstr)
+            cnx.create_entity('CWSource', name=sourceuri, type=unicode(type),
+                              config=cfgstr, parser=unicode(parser), url=unicode(url))
             cnx.commit()
 
 
@@ -596,7 +650,6 @@ class ResetAdminPasswordCommand(Command):
             sys.exit(1)
         cnx = source_cnx(sourcescfg['system'])
         driver = sourcescfg['system']['db-driver']
-        from logilab.database import get_db_helper
         dbhelper = get_db_helper(driver)
         cursor = cnx.cursor()
         # check admin exists
@@ -1098,9 +1151,14 @@ for cmdclass in (CreateInstanceDBCommand, InitInstanceCommand,
 
 db_options = (
     ('db',
-     {'short': 'd', 'type' : 'named', 'metavar' : 'key1:value1,key2:value2',
+     {'short': 'd', 'type' : 'named', 'metavar' : '[section1.]key1:value1,[section2.]key2:value2',
       'default': None,
-      'help': 'set <key> to <value> in "source" configuration file.',
+      'help': '''set <key> in <section> to <value> in "source" configuration file. If <section> is not specified, it defaults to "system".
+
+Beware that changing admin.login or admin.password using this command
+will NOT update the database with new admin credentials.  Use the
+reset-admin-pwd command instead.
+''',
       }),
     )
 
@@ -1114,10 +1172,14 @@ def configure_instance2(self, appid):
         appcfg = ServerConfiguration.config_for(appid)
         srccfg = appcfg.read_sources_file()
         for key, value in self.config.db.iteritems():
+            if '.' in key:
+                section, key = key.split('.', 1)
+            else:
+                section = 'system'
             try:
-                srccfg['system'][key] = value
+                srccfg[section][key] = value
             except KeyError:
-                raise ConfigurationError('unknown configuration key "%s" for source' % key)
+                raise ConfigurationError('unknown configuration key "%s" in section "%s" for source' % (key, section))
         admcfg = Configuration(options=USER_OPTIONS)
         admcfg['login'] = srccfg['admin']['login']
         admcfg['password'] = srccfg['admin']['password']
