@@ -128,8 +128,12 @@ class _CubicWebRequestBase(RequestSessionBase):
             self.datadir_url = vreg.config.https_datadir_url
         else:
             self.datadir_url = vreg.config.datadir_url
+        #: enable UStringIO's write tracing
+        self.tracehtml = False
+        if vreg.config.debugmode:
+            self.tracehtml = bool(form.pop('_cwtracehtml', False))
         #: raw html headers that can be added from any view
-        self.html_headers = HTMLHead(self)
+        self.html_headers = HTMLHead(self, tracewrites=self.tracehtml)
         #: received headers
         self._headers_in = Headers()
         if headers is not None:
@@ -139,6 +143,11 @@ class _CubicWebRequestBase(RequestSessionBase):
         self.setup_params(form)
         #: received body
         self.content = StringIO()
+        # prepare output header
+        #: Header used for the final response
+        self.headers_out = Headers()
+        #: HTTP status use by the final response
+        self.status_out  = 200
         # set up language based on request headers or site default (we don't
         # have a user yet, and might not get one)
         self.set_user_language(None)
@@ -152,11 +161,6 @@ class _CubicWebRequestBase(RequestSessionBase):
         #: page id, set by htmlheader template
         self.pageid = None
         self._set_pageid()
-        # prepare output header
-        #: Header used for the final response
-        self.headers_out = Headers()
-        #: HTTP status use by the final response
-        self.status_out  = 200
 
     def _set_pageid(self):
         """initialize self.pageid
@@ -179,7 +183,7 @@ class _CubicWebRequestBase(RequestSessionBase):
         self.ajax_request = value
     json_request = property(_get_json_request, _set_json_request)
 
-    def base_url(self, secure=None):
+    def _base_url(self, secure=None):
         """return the root url of the instance
 
         secure = False -> base-url
@@ -192,7 +196,7 @@ class _CubicWebRequestBase(RequestSessionBase):
         if secure:
             base_url = self.vreg.config.get('https-url')
         if base_url is None:
-            base_url = super(_CubicWebRequestBase, self).base_url()
+            base_url = super(_CubicWebRequestBase, self)._base_url()
         return base_url
 
     @property
@@ -237,7 +241,6 @@ class _CubicWebRequestBase(RequestSessionBase):
     no_script_form_params = set(('vid',
                                  'etype',
                                  'vtitle', 'title',
-                                 '__message',
                                  '__redirectvid', '__redirectrql'))
 
     def setup_params(self, params):
@@ -260,14 +263,6 @@ class _CubicWebRequestBase(RequestSessionBase):
                 val = self.no_script_form_param(param, val)
             if param == '_cwmsgid':
                 self.set_message_id(val)
-            elif param == '__message':
-                warn('[3.13] __message in request parameter is deprecated (may '
-                     'only be given to .build_url). Seeing this message usualy '
-                     'means your application hold some <form> where you should '
-                     'replace use of __message hidden input by form.set_message, '
-                     'so new _cwmsgid mechanism is properly used',
-                     DeprecationWarning)
-                self.set_message(val)
             else:
                 self.form[param] = val
 
@@ -408,6 +403,7 @@ class _CubicWebRequestBase(RequestSessionBase):
             return breadcrumbs.pop()
         return self.base_url()
 
+    @deprecated('[3.19] use a traditional ajaxfunc / controller')
     def user_rql_callback(self, rqlargs, *args, **kwargs):
         """register a user callback to execute some rql query, and return a URL
         to call that callback which can be inserted in an HTML view.
@@ -439,14 +435,11 @@ class _CubicWebRequestBase(RequestSessionBase):
         """
         self.add_js('cubicweb.ajax.js')
         jsfunc = kwargs.pop('jsfunc', 'userCallbackThenReloadPage')
-        if 'msg' in kwargs:
-            warn('[3.10] msg should be given as positional argument',
-                 DeprecationWarning, stacklevel=2)
-            args = (kwargs.pop('msg'),) + args
         assert not kwargs, 'dunno what to do with remaining kwargs: %s' % kwargs
         cbname = self.register_onetime_callback(cb, *cbargs)
         return "javascript: %s" % getattr(js, jsfunc)(cbname, *args)
 
+    @deprecated('[3.19] use a traditional ajaxfunc / controller')
     def register_onetime_callback(self, func, *args):
         cbname = build_cb_uid(func.__name__)
         def _cb(req):
@@ -457,12 +450,14 @@ class _CubicWebRequestBase(RequestSessionBase):
         self.set_page_data(cbname, _cb)
         return cbname
 
+    @deprecated('[3.19] use a traditional ajaxfunc / controller')
     def unregister_callback(self, pageid, cbname):
         assert pageid is not None
         assert cbname.startswith('cb_')
         self.info('unregistering callback %s for pageid %s', cbname, pageid)
         self.del_page_data(cbname)
 
+    @deprecated('[3.19] use a traditional ajaxfunc / controller')
     def clear_user_callbacks(self):
         if self.session is not None: # XXX
             for key in list(self.session.data):
@@ -569,7 +564,7 @@ class _CubicWebRequestBase(RequestSessionBase):
         except KeyError:
             return SimpleCookie()
 
-    def set_cookie(self, name, value, maxage=300, expires=None, secure=False):
+    def set_cookie(self, name, value, maxage=300, expires=None, secure=False, httponly=False):
         """set / update a cookie
 
         by default, cookie will be available for the next 5 minutes.
@@ -595,7 +590,7 @@ class _CubicWebRequestBase(RequestSessionBase):
             expires = None
         # make sure cookie is set on the correct path
         cookie = Cookie(str(name), str(value), self.base_url_path(),
-                        expires=expires, secure=secure)
+                        expires=expires, secure=secure, httponly=httponly)
         self.headers_out.addHeader('Set-cookie', cookie)
 
     def remove_cookie(self, name, bwcompat=None):
@@ -786,10 +781,6 @@ class _CubicWebRequestBase(RequestSessionBase):
             if 'Expires' not in self.headers_out:
                 # Expires header seems to be required by IE7 -- Are you sure ?
                 self.add_header('Expires', 'Sat, 01 Jan 2000 00:00:00 GMT')
-            if self.http_method() == 'HEAD':
-                self.status_out = 200
-                # XXX replace by True once validate_cache bw compat method is dropped
-                return 200
             # /!\ no raise, the function returns and we keep processing the request
         else:
             # overwrite headers_out to forge a brand new not-modified response
@@ -1005,6 +996,7 @@ class _CubicWebRequestBase(RequestSessionBase):
                 pass
         if vreg.config.get('language-negociation', False):
             # 2. http accept-language
+            self.headers_out.addHeader('Vary', 'Accept-Language')
             for lang in self.header_accept_language():
                 if lang in self.translations:
                     self.set_language(lang)
@@ -1082,6 +1074,10 @@ class ConnectionCubicWebRequestBase(_CubicWebRequestBase):
     set_shared_data = _cnx_func('set_shared_data')
     describe = _cnx_func('describe') # deprecated XXX
 
+    # security #################################################################
+
+    security_enabled = _cnx_func('security_enabled')
+
     # server-side service call #################################################
 
     def call_service(self, regid, **kwargs):
